@@ -1,5 +1,5 @@
 """
- Copyright (C) 2020 biqqles.
+ Copyright (C) 2020, 2021 biqqles.
  This Source Code Form is subject to the terms of the Mozilla Public
  License, v. 2.0. If a copy of the MPL was not distributed with this
  file, You can obtain one at http://mozilla.org/MPL/2.0/.
@@ -8,7 +8,7 @@
  work, as well as functions which operate on them.
 """
 from types import FunctionType as Function
-from typing import Any, Dict, Hashable, List, Optional, Type, TypeVar, Union
+from typing import Any, Dict, List, Optional, Type, TypeVar, Union
 
 DataClass = Any  # type hint for variables that should be data class instances
 
@@ -25,7 +25,7 @@ class Hint:
 
     @classmethod
     def is_hinted(cls, hint: Union[Type, str]) -> bool:
-        """Check whether a type hint represents Internal."""
+        """Check whether a type hint represents this Hint."""
         return (hasattr(hint, '__args__') and cls in hint.__args__ or
                 (type(hint) is str and f'{cls.__name__}[' in hint))
 
@@ -39,12 +39,20 @@ class Hashed(Hint):
 
 
 class DataClassMeta(type):
-    """The metaclass for a data class."""
+    """The metaclass that implements data class behaviour."""
     DEFAULT_OPTIONS = dict(init=True, repr=True, eq=True, iter=False, frozen=False, kwargs=False, slots=False,
                            order=False, unsafe_hash=True, hide_internals=True)
 
     def __new__(mcs, name, bases, dict_, **kwargs):
+        """Create a new data class."""
+
+        # delete what may become stale references so that Python creates new ones
+
+        dict_.pop('__dict__', None)
+        dict_ = {f: v for f, v in dict_.items() if type(v).__name__ != 'member_descriptor'}
+
         # collect annotations, defaults, slots and options from this class' ancestors, in definition order
+
         all_annotations = {}
         all_defaults = {}
         all_slots = set()
@@ -54,34 +62,37 @@ class DataClassMeta(type):
         dataclass_bases = [vars(b) for b in bases if hasattr(b, '__dataclass__')]
         for b in dataclass_bases + [dict_]:
             all_annotations.update(b.get('__annotations__', {}))
-            all_defaults.update({f: v for f, v in b.get('__defaults__', dict_).items() if f in all_annotations})
-            all_slots.update(b.get('__slots__', []))
+            all_defaults.update(b.get('__defaults__', {}))
+            all_slots.update(b.get('__slots__', set()))
             options.update(b.get('__dataclass__', {}))
 
+            # add user-defined methods to dict_ so that we know not to replace them. Do not replace methods
+            # that are already there as this would break overriding
+            dict_.update({f: v for f, v in b.items() if user_func(v) if f not in dict_})
+
             post_init = post_init or user_func(b.get('__init__')) or user_func(b.get('__post_init__'))
+
+        # update options and defaults for *this* class
+
         options.update(kwargs)
+        all_defaults.update({f: v for f, v in dict_.items() if f in all_annotations})
+
+        # store defaults, annotations and decorator options for future subclasses
 
-        # fill out this class' dict and store defaults, annotations and decorator options for future subclasses
-        dict_.update(all_defaults)
         dict_['__defaults__'] = all_defaults
         dict_['__annotations__'] = all_annotations
         dict_['__dataclass__'] = options
 
-        # delete what will become stale references so that Python creates new ones
-        dict_.pop('__dict__', None)
-        dict_.pop('__weakref__', None)
+        # create and apply generated methods and attributes
 
-        # create/apply generated methods and attributes
         if options['slots']:
-            # if the slots option is added, specify them. Values with default values must only be present in slots,
+            # if the slots option is added, add __slots__. Values with default values must only be present in slots,
             # not dict, otherwise Python will interpret them as read only
-            for d in all_defaults.keys():
+            for d in all_annotations.keys() & dict_.keys():
                 del dict_[d]
-            dict_['__slots__'] = all_annotations.keys() - all_slots
+            dict_['__slots__'] = tuple(all_annotations.keys() - all_slots)
         elif '__slots__' in dict_:
-            # if the slots option gets removed, remove descriptors and __slots__
-            for d in all_annotations.keys() - all_defaults.keys() & dict_.keys():
-                del dict_[d]
+            # if the slots option gets removed, remove __slots__
             del dict_['__slots__']
 
         if options['init'] and all_annotations:  # only generate __init__ if there are fields to set
@@ -106,7 +117,7 @@ class DataClassMeta(type):
             dict_.setdefault('__lt__', __lt__)
 
         if (options['eq'] and options['frozen']) or options['unsafe_hash']:
-            dict_.setdefault('__hash__', __hash__)
+            dict_.setdefault('__hash__', generate_hash(all_annotations))
 
         return super().__new__(mcs, name, bases, dict_)
 
@@ -127,7 +138,7 @@ def generate_init(annotations: Dict, defaults: Dict, user_init: bool, gen_kwargs
     class. When the data class is initialised, arguments to this function are applied to the fields of the new instance.
     A user-defined __init__, if present, must be aliased to avoid conflicting."""
     arguments = [a for a in annotations if a not in defaults]
-    default_arguments = [f'{a}={a}' for a in defaults]
+    default_arguments = [f'{a}={a}' for a in annotations if a in defaults]
     args = ['*args'] if user_init else []
     kwargs = ['**kwargs'] if user_init or gen_kwargs else []
 
@@ -153,6 +164,14 @@ def generate_init(annotations: Dict, defaults: Dict, user_init: bool, gen_kwargs
     return eval_function('__init__', lines, annotations, defaults, default_names)
 
 
+def generate_hash(annotations: Dict) -> Function:
+    """Generate a __hash__ method for a data class. The hashed value consists of a tuple of the instance's type
+    followed by any fields marked as "Hashed"."""
+    hash_of = ', '.join(['self.__class__', *(f'self.{f}' for f, h in annotations.items() if Hashed.is_hinted(h))])
+    return eval_function('__hash__', ['def __hash__(self):',
+                                      f'return hash(({hash_of}))'], dict(self=DataClass), {}, {})
+
+
 def eval_function(name: str, lines: List[str], annotations: Dict, locals_: Dict, globals_: Dict) -> Function:
     """Evaluate a function definition, returning the resulting object."""
     exec('\n\t'.join(lines), globals_, locals_)
@@ -182,11 +201,6 @@ def __lt__(self: DataClass, other: DataClass):
     return NotImplemented
 
 
-def __hash__(self):
-    # currently not ideal since gives no warning if a field expected to be hashable is not
-    return hash((type(self), *(v for v in self.__tuple__ if isinstance(v, Hashable))))
-
-
 def __iter__(self):
     return iter(self.__tuple__)
 
